@@ -1,7 +1,7 @@
 /*
  * dns.h -- DNS definitions.
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -14,6 +14,11 @@ enum rr_section {
 	QUESTION_SECTION,
 	ANSWER_SECTION,
 	AUTHORITY_SECTION,
+	/*
+	 * Use a split authority section to ensure that optional
+	 * NS RRsets in the response can be omitted.
+	 */
+	OPTIONAL_AUTHORITY_SECTION,
 	ADDITIONAL_SECTION,
 	/*
 	 * Use a split additional section to ensure A records appear
@@ -168,6 +173,7 @@ enum rdata_wireformat
 	RDATA_WF_SHORT,		     /* 16-bit integer.  */
 	RDATA_WF_LONG,		     /* 32-bit integer.  */
 	RDATA_WF_TEXT,		     /* Text string.  */
+	RDATA_WF_TEXTS,		     /* Text string sequence.  */
 	RDATA_WF_A,		     /* 32-bit IPv4 address.  */
 	RDATA_WF_AAAA,		     /* 128-bit IPv6 address.  */
 	RDATA_WF_BINARY, 	     /* Binary data (unknown length).  */
@@ -185,6 +191,7 @@ enum rdata_zoneformat
 	RDATA_ZF_DNAME,		/* Domain name.  */
 	RDATA_ZF_LITERAL_DNAME,	/* DNS name (not lowercased domain name).  */
 	RDATA_ZF_TEXT,		/* Text string.  */
+	RDATA_ZF_TEXTS,		/* Text string sequence.  */
 	RDATA_ZF_BYTE,		/* 8-bit integer.  */
 	RDATA_ZF_SHORT,		/* 16-bit integer.  */
 	RDATA_ZF_LONG,		/* 32-bit integer.  */
@@ -215,8 +222,8 @@ struct rrtype_descriptor
 	uint16_t    type;	/* RR type */
 	const char *name;	/* Textual name.  */
 	int         token;	/* Parser token.  */
-	uint8_t     minimum;	/* Minimum number of RDATAs.  */
-	uint8_t     maximum;	/* Maximum number of RDATAs.  */
+	uint32_t    minimum;	/* Minimum number of RDATAs.  */
+	uint32_t    maximum;	/* Maximum number of RDATAs.  */
 	uint8_t     wireformat[MAXRDATALEN]; /* rdata_wireformat_type */
 	uint8_t     zoneformat[MAXRDATALEN]; /* rdata_zoneformat_type  */
 };
