@@ -1,7 +1,7 @@
 /*
- * edns.h -- EDNS definitions (RFC 2671).
+ * edns.c -- EDNS definitions (RFC 2671).
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -71,7 +71,7 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 	uint16_t opt_nsid;
 
 	edns->position = buffer_position(packet);
-	
+
 	if (!buffer_available(packet, (OPT_LEN + OPT_RDATA)))
 		return 0;
 
@@ -82,14 +82,14 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 		buffer_set_position(packet, edns->position);
 		return 0;
 	}
-	
+
 	opt_class = buffer_read_u16(packet);
 	opt_extended_rcode = buffer_read_u8(packet);
 	opt_version = buffer_read_u8(packet);
 	opt_flags = buffer_read_u16(packet);
 	opt_rdlen = buffer_read_u16(packet);
-	
-	if (opt_rdlen != 0 || opt_version != 0) {
+
+	if (opt_version != 0) {
 		edns->status = EDNS_ERROR;
 		return 1;
 	}
@@ -98,7 +98,7 @@ edns_parse_record(edns_record_type *edns, buffer_type *packet)
 		/* there is more to come, read opt code
 		 * should be NSID - there are no others */
 		opt_nsid = buffer_read_u16(packet);
-		edns->nsid = (opt_nsid & NSID_MASK);
+		edns->nsid = (opt_nsid == NSID_CODE);
 		/* extra check for the value */
 	}
 
