@@ -1,7 +1,7 @@
 /*
  * ipc.c - Interprocess communication routines. Handlers read and write.
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -79,7 +79,7 @@ child_handle_parent_command(netio_type *ATTR_UNUSED(netio),
 {
 	sig_atomic_t mode;
 	int len;
-	struct ipc_handler_conn_data *data = 
+	struct ipc_handler_conn_data *data =
 		(struct ipc_handler_conn_data *) handler->user_data;
 	if (!(event_types & NETIO_EVENT_READ)) {
 		return;
@@ -116,7 +116,7 @@ child_handle_parent_command(netio_type *ATTR_UNUSED(netio),
 	}
 
 	switch (mode) {
-	case NSD_STATS: 
+	case NSD_STATS:
 	case NSD_QUIT:
 		data->nsd->mode = mode;
 		break;
@@ -141,7 +141,7 @@ parent_handle_xfrd_command(netio_type *ATTR_UNUSED(netio),
 {
 	sig_atomic_t mode;
 	int len;
-	struct ipc_handler_conn_data *data = 
+	struct ipc_handler_conn_data *data =
 		(struct ipc_handler_conn_data *) handler->user_data;
 	if (!(event_types & NETIO_EVENT_READ)) {
 		return;
@@ -165,14 +165,14 @@ parent_handle_xfrd_command(netio_type *ATTR_UNUSED(netio),
 		data->conn->is_reading = 0;
 		buffer_flip(data->conn->packet);
 		zone = handle_xfrd_zone_state(data->nsd, data->conn->packet);
-		if(!zone) 
+		if(!zone)
 			return;
 		/* forward to all children */
 		for (i = 0; i < data->nsd->child_count; ++i) {
 			if(!zone->dirty[i]) {
 				zone->dirty[i] = 1;
 				stack_push(data->nsd->children[i].dirty_zones, zone);
-				data->nsd->children[i].handler->event_types |= 
+				data->nsd->children[i].handler->event_types |=
 					NETIO_EVENT_WRITE;
 			}
 		}
@@ -232,7 +232,7 @@ write_zone_state_packet(buffer_type* packet, zone_type* zone)
 	buffer_write(packet, &cmd, sizeof(cmd));
 	buffer_write(packet, &sz, sizeof(sz));
 	buffer_write(packet, &ok, sizeof(ok));
-	buffer_write(packet, domain_dname(zone->apex), 
+	buffer_write(packet, domain_dname(zone->apex),
 		dname_total_size(domain_dname(zone->apex)));
 	buffer_flip(packet);
 }
@@ -265,7 +265,7 @@ debug_print_fwd_name(int ATTR_UNUSED(len), buffer_type* packet, int acl_num)
 	buffer_skip(packet, 12);
 	if(packet_read_query_section(packet, qnamebuf, &qtype, &qclass)) {
 		dname = dname_make(tempregion, qnamebuf, 1);
-		log_msg(LOG_INFO, "main: fwd packet for %s, acl %d", 
+		log_msg(LOG_INFO, "main: fwd packet for %s, acl %d",
 			dname_to_string(dname,0), acl_num);
 	} else {
 		log_msg(LOG_INFO, "main: fwd packet badqname, acl %d", acl_num);
@@ -286,7 +286,7 @@ send_quit_to_child(struct main_ipc_handler_data* data, int fd)
 		return;
 	}
 	data->child->need_to_send_QUIT = 0;
-	DEBUG(DEBUG_IPC,2, (LOG_INFO, "main: sent quit to child %d", 
+	DEBUG(DEBUG_IPC,2, (LOG_INFO, "main: sent quit to child %d",
 		(int)data->child->pid));
 }
 
@@ -297,9 +297,9 @@ parent_handle_child_command(netio_type *ATTR_UNUSED(netio),
 {
 	sig_atomic_t mode;
 	int len;
-	struct main_ipc_handler_data *data = 
+	struct main_ipc_handler_data *data =
 		(struct main_ipc_handler_data*)handler->user_data;
-	
+
 	/* do a nonblocking write to the child if it is ready. */
 	if (event_types & NETIO_EVENT_WRITE) {
 		if(!data->busy_writing_zone_state &&
@@ -350,12 +350,12 @@ parent_handle_child_command(netio_type *ATTR_UNUSED(netio),
 	if (data->forward_mode) {
 		int got_acl;
 		/* forward the data to xfrd */
-		DEBUG(DEBUG_IPC,2, (LOG_INFO, 
+		DEBUG(DEBUG_IPC,2, (LOG_INFO,
 			"main passed packet readup %d", (int)data->got_bytes));
 		if(data->got_bytes < sizeof(data->total_bytes))
 		{
-			if ((len = read(handler->fd, 
-				(char*)&data->total_bytes+data->got_bytes, 
+			if ((len = read(handler->fd,
+				(char*)&data->total_bytes+data->got_bytes,
 				sizeof(data->total_bytes)-data->got_bytes)) == -1) {
 				log_msg(LOG_ERR, "handle_child_command: read: %s",
 					strerror(errno));
@@ -413,7 +413,7 @@ parent_handle_child_command(netio_type *ATTR_UNUSED(netio),
 		data->got_bytes += len;
 		if(got_acl >= (int)sizeof(data->acl_num)) {
 			uint16_t len = htons(data->total_bytes);
-			DEBUG(DEBUG_IPC,2, (LOG_INFO, 
+			DEBUG(DEBUG_IPC,2, (LOG_INFO,
 				"main fwd passed packet write %d", (int)data->got_bytes));
 #ifndef NDEBUG
 			if(nsd_debug_level >= 2)
@@ -423,9 +423,9 @@ parent_handle_child_command(netio_type *ATTR_UNUSED(netio),
 			mode = NSD_PASS_TO_XFRD;
 			if(!write_socket(*data->xfrd_sock, &mode, sizeof(mode)) ||
 			   !write_socket(*data->xfrd_sock, &len, sizeof(len)) ||
-			   !write_socket(*data->xfrd_sock, buffer_begin(data->packet), 
+			   !write_socket(*data->xfrd_sock, buffer_begin(data->packet),
 				data->total_bytes) ||
-			   !write_socket(*data->xfrd_sock, &data->acl_num, 
+			   !write_socket(*data->xfrd_sock, &data->acl_num,
 			   	sizeof(data->acl_num))) {
 				log_msg(LOG_ERR, "error in ipc fwd main2xfrd: %s",
 					strerror(errno));
@@ -448,7 +448,7 @@ parent_handle_child_command(netio_type *ATTR_UNUSED(netio),
 			if(data->nsd->children[i].child_fd == handler->fd) {
 				data->nsd->children[i].child_fd = -1;
 				data->nsd->children[i].has_exited = 1;
-				DEBUG(DEBUG_IPC,1, (LOG_INFO, 
+				DEBUG(DEBUG_IPC,1, (LOG_INFO,
 					"server %d closed cmd channel",
 					(int) data->nsd->children[i].pid));
 			}
@@ -461,7 +461,7 @@ parent_handle_child_command(netio_type *ATTR_UNUSED(netio),
 	case NSD_QUIT:
 		data->nsd->mode = mode;
 		break;
-	case NSD_STATS: 
+	case NSD_STATS:
 		data->nsd->signal_hint_stats = 1;
 		break;
 	case NSD_REAP_CHILDREN:
@@ -484,7 +484,7 @@ void
 parent_check_all_children_exited(struct nsd* nsd)
 {
 	size_t i;
-	for(i=0; i<nsd->child_count; i++) {
+	for(i=0; i < nsd->child_count; i++) {
 		if(!nsd->children[i].need_to_exit)
 		      return;
 		if(!nsd->children[i].has_exited)
@@ -525,12 +525,12 @@ parent_handle_reload_command(netio_type *ATTR_UNUSED(netio),
 	case NSD_QUIT_SYNC:
 		/* set all children to exit, only then notify xfrd. */
 		/* so that buffered packets to pass to xfrd can arrive. */
-		for(i=0; i<nsd->child_count; i++) {
+		for(i=0; i < nsd->child_count; i++) {
 			nsd->children[i].need_to_exit = 1;
 			if(nsd->children[i].pid > 0 &&
 			   nsd->children[i].child_fd > 0) {
 				nsd->children[i].need_to_send_QUIT = 1;
-				nsd->children[i].handler->event_types 
+				nsd->children[i].handler->event_types
 					|= NETIO_EVENT_WRITE;
 			} else {
 				if(nsd->children[i].child_fd == -1)
@@ -546,7 +546,7 @@ parent_handle_reload_command(netio_type *ATTR_UNUSED(netio),
 	}
 }
 
-static void 
+static void
 xfrd_write_expire_notification(buffer_type* buffer, xfrd_zone_t* zone)
 {
 	sig_atomic_t cmd = NSD_ZONE_STATE;
@@ -556,7 +556,7 @@ xfrd_write_expire_notification(buffer_type* buffer, xfrd_zone_t* zone)
 	if(zone->state == xfrd_zone_expired)
 		ok = 0;
 
-	DEBUG(DEBUG_IPC,1, (LOG_INFO, 
+	DEBUG(DEBUG_IPC,1, (LOG_INFO,
 		"xfrd encoding ipc zone state msg for zone %s state %d.",
 		zone->apex_str, (int)zone->state));
 
@@ -580,7 +580,7 @@ xfrd_send_reload_req(xfrd_state_t* xfrd)
 			strerror(errno));
 		return;
 	}
-	DEBUG(DEBUG_IPC,1, (LOG_ERR, "xfrd: asked nsd to reload new updates"));
+	DEBUG(DEBUG_IPC,1, (LOG_INFO, "xfrd: asked nsd to reload new updates"));
 	xfrd_prepare_zones_for_reload();
 	xfrd->reload_cmd_last_sent = xfrd_time();
 	xfrd->need_to_send_reload = 0;
@@ -602,7 +602,7 @@ xfrd_send_quit_req(xfrd_state_t* xfrd)
 	xfrd->need_to_send_quit = 0;
 }
 
-static void 
+static void
 xfrd_handle_ipc_SOAINFO(xfrd_state_t* xfrd, buffer_type* packet)
 {
 	xfrd_soa_t soa;
@@ -617,7 +617,7 @@ xfrd_handle_ipc_SOAINFO(xfrd_state_t* xfrd, buffer_type* packet)
 
 	if(!buffer_available(packet, sizeof(uint32_t)*6 + sizeof(uint8_t)*2)) {
 		/* NSD has zone without any info */
-		DEBUG(DEBUG_IPC,1, (LOG_INFO, "SOAINFO for %s lost zone", 
+		DEBUG(DEBUG_IPC,1, (LOG_INFO, "SOAINFO for %s lost zone",
 			dname_to_string(dname,0)));
 		soa_ptr = NULL;
 	} else {
@@ -636,20 +636,20 @@ xfrd_handle_ipc_SOAINFO(xfrd_state_t* xfrd, buffer_type* packet)
 		if(!buffer_available(packet, soa.email[0]))
 			return;
 		buffer_read(packet, soa.email+1, soa.email[0]);
-	
+
 		soa.serial = htonl(buffer_read_u32(packet));
 		soa.refresh = htonl(buffer_read_u32(packet));
 		soa.retry = htonl(buffer_read_u32(packet));
 		soa.expire = htonl(buffer_read_u32(packet));
 		soa.minimum = htonl(buffer_read_u32(packet));
-		DEBUG(DEBUG_IPC,1, (LOG_INFO, "SOAINFO for %s %u", 
+		DEBUG(DEBUG_IPC,1, (LOG_INFO, "SOAINFO for %s %u",
 			dname_to_string(dname,0), ntohl(soa.serial)));
 	}
 
 	if(!zone) {
 		DEBUG(DEBUG_IPC,1, (LOG_INFO, "xfrd: zone %s master zone updated",
 			dname_to_string(dname,0)));
-		notify_handle_master_zone_soainfo(xfrd->notify_zones, 
+		notify_handle_master_zone_soainfo(xfrd->notify_zones,
 			dname, soa_ptr);
 		return;
 	}
@@ -657,8 +657,8 @@ xfrd_handle_ipc_SOAINFO(xfrd_state_t* xfrd, buffer_type* packet)
 }
 
 void
-xfrd_handle_ipc(netio_type* ATTR_UNUSED(netio), 
-	netio_handler_type *handler, 
+xfrd_handle_ipc(netio_type* ATTR_UNUSED(netio),
+	netio_handler_type *handler,
 	netio_event_types_type event_types)
 {
 	xfrd_state_t* xfrd = (xfrd_state_t*)handler->user_data;
@@ -713,7 +713,7 @@ xfrd_handle_ipc(netio_type* ATTR_UNUSED(netio),
 	}
 
 }
-	
+
 static void
 xfrd_handle_ipc_read(netio_handler_type *handler, xfrd_state_t* xfrd)
 {
@@ -767,7 +767,7 @@ xfrd_handle_ipc_read(netio_handler_type *handler, xfrd_state_t* xfrd)
 		}
 		return;
 	}
-        
+
         if((len = read(handler->fd, &cmd, sizeof(cmd))) == -1) {
                 log_msg(LOG_ERR, "xfrd_handle_ipc: read: %s",
                         strerror(errno));
@@ -791,7 +791,7 @@ xfrd_handle_ipc_read(netio_handler_type *handler, xfrd_state_t* xfrd)
 		DEBUG(DEBUG_IPC,1, (LOG_INFO, "xfrd: ipc recv SOA_BEGIN"));
 		/* reload starts sending SOA INFOs; don't block */
 		xfrd->parent_soa_info_pass = 1;
-		/* reset the nonblocking ipc write; 
+		/* reset the nonblocking ipc write;
 		   the new parent does not want half a packet */
 		xfrd->sending_zone_state = 0;
 		break;
@@ -808,6 +808,7 @@ xfrd_handle_ipc_read(netio_handler_type *handler, xfrd_state_t* xfrd)
 		xfrd->parent_soa_info_pass = 0;
 		xfrd->ipc_send_blocked = 0;
 		handler->event_types |= NETIO_EVENT_WRITE;
+		xfrd_reopen_logfile();
 		xfrd_check_failed_updates();
 		xfrd_send_expy_all_zones();
 		break;
