@@ -281,7 +281,10 @@ xfrd_read_state(struct xfrd_state* xfrd)
 		zone->soa_disk = soa_disk_read;
 		zone->soa_notified = soa_notified_read;
 		zone->soa_nsd_acquired = soa_nsd_acquired_read;
-		zone->soa_disk_acquired = soa_disk_acquired_read;
+		/* we had better use what we got from starting NSD, not
+		 * what we store in this file, because the actual zone
+		 * contents trumps the contents of this cache */
+		/* zone->soa_disk_acquired = soa_disk_acquired_read; */
 		zone->soa_notified_acquired = soa_notified_acquired_read;
 		xfrd_handle_incoming_soa(zone, &incoming_soa, incoming_acquired);
 	}
