@@ -1,7 +1,7 @@
 /*
  * zonec.h -- zone compiler.
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -108,6 +108,8 @@ void parse_unknown_rdata(uint16_t type, uint16_t *wireformat);
 
 uint32_t zparser_ttl2int(const char *ttlstr, int* error);
 void zadd_rdata_wireformat(uint16_t *data);
+void zadd_rdata_txt_wireformat(uint16_t *data, int first);
+void zadd_rdata_txt_clean_wireformat();
 void zadd_rdata_domain(domain_type *domain);
 
 void set_bitnsec(uint8_t  bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
